#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import commands
import sys
from optparse import OptionParser

__version__ = "1.0"

def parse_command_line():
    parser = OptionParser(__doc__, version='%%prog version %s' % __version__)
    parser.add_option('-n', '--number', action='store', dest='num_ve', type='int',
                      help='numbre de VE', default=1)
    parser.add_option('-p', '--pidsmax', action='store', dest='max_total_pids', type='int',
                      help='numbre de pids maximum', default=8000)
    parser.add_option('-m', '--memory', action='store', dest='mem_total', type='int',
                      help='mémoire à partager (en octet)', default=-1)
    parser.add_option('-v', '--veid', action='store', dest='veid', type='string',
                      help='veid de l\'hôte', default='-1') 
    parser.add_option('-c', '--cpuunits', action='store', dest='cpuunits', type='int',
                      help='cpuunits pour l\'hôte', default=-1)
    options, args = parser.parse_args()
    if len(args) >= 1:
        parser.error("le script ne prend pas d'argument")

    return options, args

def run(cmd, noexit=False):
    """raccourci d'execution d'une commande shell"""
    ret, res = commands.getstatusoutput(cmd)
    if ret != 0 and noexit == False:
        print "\033[31mLa commande %s a echouee\033[0m" % cmd
        sys.exit(res)
    return res
def apply_lim (name, bar, lim):
    bar=min(int(bar), MAXVAL)
    lim=min(int(lim), MAXVAL)
    print "nom %s - bar : %s - lim : %s" % (name, bar, lim)
    run("vzctl set %s --%s %s:%s --save" % (veid, name, bar, lim))

#memoire totale
mem_total=run('cat /proc/meminfo|grep ^MemTotal:|cut -d: -f2|sed \'s/^ *//g\'').split()
if mem_total[1] == 'kB':
    mem_total=int(mem_total[0])*1024
else:
    mem_total=int(mem_total[0])
low_total=run('cat /proc/meminfo|grep ^LowTotal:|cut -d: -f2|sed \'s/^ *//g\'').split()
if low_total[1] == 'kB':
    low_total=int(low_total[0])*1024
else:
    low_total=int(low_total[0])

SYSRSRV=52428800
MIN_GUARPG=1024
MINPROC=30
K_KMEM_MAX=2.0
MAXVAL=2147483647
KMEM_DELTA=1.1
MAXIPTENT=2000
IPTENT_PVE=200
NFILE_AVNPROC=32
DCACHE_DELTA=1.03
NFLOCK_NFILE=0.10
FLOCK_PVE=1000
NFLOCK_DELTA=1.1
PGLOCK_KMEM=204800
MIN_PGLOCK=4
PTY_PROC=0.1
MIN_PTY=4
PTY_PVE=512
PRIVVM_PVE=0.6/4096
SHMPG_PRIVVM=0.10
PRIVVM_DELTA=1.1
MIN_IPTENT=4

cpuunits=1000

options, args = parse_command_line()
num_ve=options.num_ve
max_total_pids=options.max_total_pids
if options.veid == '-1':
    print 'veid needed'
    sys.exit()
veid=options.veid

if options.mem_total != -1:
    mem_total=options.mem_total

if options.cpuunits != -1:
    cpuunits=options.cpuunits

mem_total=int(mem_total*0.6)

proc_calc=int(run('cat /proc/sys/kernel/threads-max').split()[0])
if low_total > 2147483648:
    if proc_calc > 2 * max_total_pids:
        proc_calc = 2 * max_total_pids
elif proc_calc > max_total_pids:
    proc_calc = max_total_pids
#la taille swap
swap_total=int(run('lvdisplay -c|grep %s-swap|awk -F: \'{ print 4096*$8-8 }\'' % veid))
swap_total=int(swap_total*1024)
quota="no"

pagesize=4096
avnproc = tcpbuf = 0

k_kmem=[1, 1.2, 1.8]
k_nproc=[1, 1.5, 2]
k_msl=[10485760, 2097152, 0]
k_kpp=[81920, 65536, 53248]
k_avnpr=[8, 5, 3]
k_sock1=[262144, 131072, 65536]
k_sock2=[4096, 3036, 2560]
k_dcache=[1.5 * 384, 1.2 * 384, 1 * 384]
k_pglock=[10, 3, 1]
numsiginfo=[1024, 512, 256]
k_privvm=[6, 3, 1.5]

def main():
    # numproc : le nombre maximum de processus et thread qu'il est possible de créer.
    # avnumproc : le nombre moyen de processus et thread.
    # numtcpsock : le nombre de connexion TCP (type PF_INET family et SOCK_STREAM) en parallele.
    # numothersock : le nombre de connexion autre que TCP, comme connexion local, UDP, ...
    # vmguarpages : le nombre de page (de 4Kb) garantie, le conteneur sera capable d'allouer cet espace mémoire.
    # kmemsize : mémoire noyau. La mémoire noyau conserve les informations des processus et ne peux pas etre swappé.
    # tcpsndbuf : la taille total de mémoire kernel alloué pour les données envoyées par une application avant d'être envoyé au serveur dans les sockets TCP.
    # tcprcvbuf : la taille total de la mémoire kernel alloué pour les données reçues par le serveur avant d'être lu par l'application dans les sockets TCP.
    # othersockbuf : la taille total du buffer des sockets locaux, UDP, ... envoyées.
    # dgramrcvbuf : la taille total du buffer des sockets locaux, UDP, ... reçues.
    # oomguarpages : aucun processus ne pourra être killé si la mémoire décent en dessous du nombre de page (4Kb) alloué.
    # privvmpages : la mémoire virtuel privée (RAM + swap) alloué mais pas forcement utilisé (chaque page fait 4Kb).
    # lockedpages : la mémoire, en page (4Kb), qui ne peut pas être swappé.
    # shmpages : la taille total, en page (4Kb) de la mémoire partagée (inclus IPC, shared anonymous mappings and tmpfs objects) alloué par les processus
    # numfile : le nombre de fichiers ouverts par tous les processus.
    # numflock : le nombre de fichiers locks par tous les processus.
    # dcachesize : la taille total des dentry et inode locké en mémoire.
    # physpages : pages physiques : page mémoire vraiment utilisé.
    # numpty : le nombre de pseudo-terminal (comme une session ssh, un screen ou un xterm).
    # numsiginfo : le nombre de structures siginfo (essentiellement, cd parametre limite la taille des signaux delivery queue). 
    # numiptent : le nombre de paquet IP filtré.

    tot_pve = (mem_total + swap_total - SYSRSRV) / num_ve
    for osl in range(len(k_msl)):
        numproc = k_nproc[osl] * proc_calc / num_ve
        low_pve = low_total * 0.4 * k_kmem[osl] / num_ve
        guarpg = (tot_pve - low_pve) / pagesize
        if guarpg < MIN_GUARPG:
            guarpg = MIN_GUARPG
            low_pve = tot_pve - guarpg * pagesize
        kmem = low_pve / 2
        if low_pve < k_msl[osl]:
            continue
        if numproc < MINPROC:
            numproc=MINPROC
        avnproc = kmem / k_kpp[osl]
        if avnproc < MINPROC/2:
            continue
        if numproc < 2 * avnproc:
    	    if osl < MAX_SL:
                avnproc = numproc / 2
        if numproc <  avnproc:
            numproc = avnproc
        if numproc > avnproc * k_avnpr[osl]:
            numproc = avnproc * k_avnpr[osl]
        sockbuf = low_pve - kmem
        tcpbuf = sockbuf / 3
        if tcpbuf < k_sock1[osl] + k_sock2[osl] * numproc:
            continue
        sl = osl
        break
    if osl == len(k_msl)+1:
        print "ERROR"
        sys.exit()

    delta = k_sock2[sl] * numproc
    rest = tcpbuf - delta
    apply_lim('tcpsndbuf', rest, tcpbuf)
    apply_lim('tcprcvbuf', rest, tcpbuf)
    apply_lim('dgramrcvbuf', rest / 2, rest / 2)
    apply_lim('othersockbuf', rest / 2, rest / 2 + delta)
    apply_lim('kmemsize', kmem, kmem * KMEM_DELTA)
    apply_lim('numproc', numproc, numproc)
    apply_lim('avnumproc', avnproc, avnproc)
    apply_lim('numtcpsock', numproc, numproc)
    apply_lim('numothersock', numproc, numproc)


    iptent = MAXIPTENT / num_ve
    if iptent > IPTENT_PVE:
        iptent = IPTENT_PVE
    elif iptent < MIN_IPTENT:
        iptent = MIN_IPTENT
    apply_lim('numiptent', iptent, iptent)

    numfile = avnproc * NFILE_AVNPROC
    apply_lim('numfile',numfile,numfile)

    dcache = numfile * k_dcache[sl]
    apply_lim('dcachesize',dcache / DCACHE_DELTA,dcache)

    numflock = numfile * NFLOCK_NFILE
    if numflock > FLOCK_PVE:
        numflock = FLOCK_PVE
    apply_lim('numflock',numflock,numflock * NFLOCK_DELTA)

    lockedpages = kmem * k_pglock[sl] / PGLOCK_KMEM
    if lockedpages < MIN_PGLOCK:
        lockedpages = MIN_PGLOCK
    apply_lim('lockedpages',lockedpages,lockedpages)

    numpty = numproc * PTY_PROC
    if numpty < MIN_PTY:
        numpty = MIN_PTY
    if numpty > PTY_PVE:
        numpty = PTY_PVE
    apply_lim('numpty',numpty,numpty)

    apply_lim('numsiginfo',numsiginfo[sl],numsiginfo[sl])

    apply_lim('physpages',0,MAXVAL)

    privvm = guarpg * k_privvm[sl]
    if privvm > PRIVVM_PVE * mem_total:
        privvm = PRIVVM_PVE * mem_total
        if guarpg > privvm:
            guarpg = privvm
    apply_lim('privvmpages',privvm, privvm * PRIVVM_DELTA)
    apply_lim('vmguarpages', guarpg, MAXVAL)
    apply_lim('oomguarpages', guarpg, MAXVAL)

    shmpg = privvm * SHMPG_PRIVVM
    apply_lim('shmpages',shmpg,shmpg)
    if quota == "yes":
        space = run("df /var/lib/vz/private/%s|tail -n 1|awk '{ print $1 }'" % veid)
        inode = run("df -i /var/lib/vz/private/%s|tail -n 1|awk '{ print $1 }'" % veid)
        run('echo "DISK_QUOTA=\"yes\"" >> /etc/vz/conf/%s.conf' % veid)
        run('vzctl set %s --diskspace %s:%s --save' % (veid, space, space))
        run('vzctl set %s --diskinodes %s:%s --save' % (veid, inode, inode))
        run('vzctl set %s --quotatime 0 --save' % veid)
        run('vzctl set %s --quotaugidlimit 5000 --save' % veid)

    # CPU
    run('vzctl set %s --cpuunits %s --save' % (veid, cpuunits))
if __name__ == '__main__':
    main()

